/*
 * can_transfer.h
 *
 * Created: 12.02.2012 11:29:06
 *  Author: Privat
 */ 


#ifndef CAN_TRANSFER_H_
#define CAN_TRANSFER_H_

#include <avr/io.h>
#include "can.h"
#include "utils.h"

// CAN data transfer
//			7		6		5		4		3		2		1		0
//	0	|						Board Identifier						|
//	1	| Type	|				Command									| 
//	2	|						Message Counter							|	
//	3	| SoB	|				Following Messages Count				|
//	4	|						Data (Byte 0)							|
//	5	|						Data (Byte 1)							|
//	6	|						Data (Byte 2)							|
//	7	|						Data (Byte 3)							|

#define CAN_HEADER_SIZE			0x04
#define CAN_DATA_SIZE			0x04
#define CAN_BUFFER_SIZE			15

#define	COMMAND_MASK			0x3F
#define	START_OF_MESSAGE_MASK	0x80


#define CAN_WHO_IS_ONLINE			0x100
#define CAN_RESET_SLAVE				0x500
#define CAN_UPDATE_SLAVE_RESPONSE	0x7FE
#define CAN_UPDATE_SLAVE_FLASH		0x7FF

typedef struct
{
	uint8_t		Board_Identifier;	// CAN Board Identifier
	uint8_t		T_OR_C;				// Type OR Command
	uint16_t	CAN_MessageID;		// CAN Message ID
} CAN_HEADER;

typedef struct
{
	uint8_t		Count;
	can_t		canmsg[CAN_BUFFER_SIZE];
} CAN_BUFFER;

typedef enum {
	// every bootloader type has this commands
	IDENTIFY		= 1,
	SET_ADDRESS		= 2,
	DATA			= 3,
	START_APP		= 4,
	
	// only avilable in the "bigger" versions
	GET_FUSEBITS	= 5,
	CHIP_ERASE		= 6,
	
	// addon Protocol CMDs
	WHO_IS_ONLINE	= 7,				// CMD check which nodes are online
	IRMP_DECODED	= 8,				// CMD IRMP new code
	TSENSOR_TEMP	= 9,				// CMD onewire temperature values
	RESET_SLAVE		= 0x0A,				// CMD to reset a specified slave
	
	REQUEST					= 0x00,
	SUCCESSFULL_RESPONSE	= 0x40,
	ERROR_RESPONSE			= 0x80,
	WRONG_NUMBER_REPSONSE	= 0xC0,
	
	NO_MESSAGE		= 0x3f
} command_t;

extern CAN_BUFFER can_buffer;

/**
 *  @brief   Will init the CAN bus
 *  @param   bitrate 
 *  @param   can filter  
 *  @return  none
 */
extern void InitCANBus(uint8_t bitrate, const prog_uint8_t *);

/**
 *  @brief   Check if a CAN Transfer is active
 *  @param   none   
 *  @return  true if transfer is active
 */
extern uint8_t CANTranserActive(void);

/**
 *  @brief   Will wait for received CAN Message for a specified NodeID
 *  @param   NodeID waiting for 
 *  @param   Pointer to a can message counter
 *  @param   Pointer to a buffer[]
 *  @param   Pointer to a uint8 len buffer[]  
 *  @return  Type of the CAN result
 */
extern uint8_t WaitforCANResponse(CAN_HEADER *, uint8_t *, uint8_t data[], uint8_t *);

/**
 *  @brief   Will receive a CAN Message
 *  @param   Pointer to a can message counter
 *  @param   Pointer to a buffer[]
 *  @param   Pointer to a uint8 len buffer[]  
 *  @return  true if a message got complete received
 */
extern uint8_t ReceiveCANMessage(CAN_HEADER *, uint8_t *, uint8_t data[], uint8_t *);

/**
 *  @brief   Will build the CAN Message(s)
 *  @param   Receiver_NodeID: define receiver NodeID
 *  @param   CAN_Command: Basic CAN Message ID
 *  @param   T_OR_C: Type OR Command
 *  @param   Message_Counter: actual message counter for this Receiver_NodeID
 *  @param   data[]: pointer to data what should be transmitted
 *  @param   len: length of data[]  
 *  @return  none
 */
extern uint8_t SendCANMessage(CAN_HEADER *, uint8_t *Message_Counter, uint8_t data[], uint8_t len);

#endif /* CAN_TRANSFER_H_ */